#!/usr/bin/python3

FS = 1000000
FE = 10000000
PTS = 10
VPPCh1=1
VPPCh2=1
VDIVDUTCh = "200mV"
VDIVREFCh = "200mV"
SLEEPTIME = 1 #Adjust to your needs. 

from tinysa_htk import *

import math
import sys
import io
import os
import time
import datetime
import getopt
import numpy as np


def parseArgs():
	global FS, FE, PTS, DryRun, LoadResistor, SignalGenerator, SystemInfo
	global NoAutoVolt, OneChannel, DUTCh, REFCh, AdjustmentFile, Logamode
	argumentList = sys.argv[1:]

	# Options
	options = "hs:e:p:dlg:oinc:f:"

	# Long options
	long_options = [
		"help",
		"FStart=",
		"FEnd=",
		"Points=",
		"DryRun",
		"LoadResistor",
		"SignalGenerator",
		"SystemInfo",
		"NoAutoVolt",
		"ChannelSetup, AdjustmentFile",
	]

	try:
		# Parsing argument
		arguments, values = getopt.getopt(argumentList, options, long_options)

		# checking each argument
		for currentArgument, currentValue in arguments:

			if currentArgument in ("-h", "--help"):
				print("s; FStart=<Start Freq. f or fk or fM>")
				print("e; FEnd=<End Freq. f or fk or fM>")
				print("p; Points=<Number of measurements> optional add:<L> logarithmic")
				print("c; ChannelSetup=<d0 / 0d / dr / rd> [d:DUT, r:REF, 0: One-channel mode]")
				print("g; SignalGenerator=<type> [default: internal; options: fy3200, tinysa]")				
				print("l; LoadResistor [50Ohm dummy load?]")
				print("d; DryRun [Do not save antything]")
				print("i; SystemInfo [Also synchronizes time]")
				print("n; NoAutoVolt [Do not set V/Div and SG amplitude]")
				print("f; AdjustmentFile=<Filename>")
				sys.exit()
			if currentArgument in ("-s", "--FStart"):
				FS = StringToHz(currentValue)
				if FS < 10:
					FS = 10
			if currentArgument in ("-e", "--FEnd"):
				FE = StringToHz(currentValue)
			if currentArgument in ("-p", "--Points"):
				if "L" in currentValue:
					Logamode = True
					PTS = int(currentValue.partition("L")[0])
				else:	
					PTS = int(currentValue)
			if currentArgument in ("-d", "--DryRun"):
				DryRun = True
			if currentArgument in ("-l", "--LoadResistor"):
				LoadResistor = True
			if currentArgument in ("-g", "--SignalGenerator"):
				SignalGenerator = currentValue
			if currentArgument in ("-o", "--OneChannel"):
				OneChannel = True
			if currentArgument in ("-i", "--SystemInfo"):
				SystemInfo = True
			if currentArgument in ("-n", "--NoAutoVolt"):
				NoAutoVolt = True
			if currentArgument in ("-c", "--ChannelSetup"):
				if currentValue == "d0":
					DUTCh = 1
					REFCh = 0
					OneChannel = True
				elif currentValue == "0d":
					DUTCh = 2
					REFCh = 0
					OneChannel = True
				elif currentValue == "dr":
					DUTCh = 1
					REFCh = 2
					OneChannel = False
				elif currentValue == "rd":
					DUTCh = 2
					REFCh = 1
					OneChannel = False
			if currentArgument in ("-f", "--AdjustmentFile"):
				AdjustmentFile = currentValue

	except getopt.error as err:
		# output error, and return with an error code
		print(str(err))


def dumpHantekInfo(han):
	print("OSCILLOSCOPE INFO")
	print("-----------------")
	print("Model: " + han.get("PRIVate:SET:Model?"), end="")
	print("Serial: " + han.get("PRIVate:SET:SERIal?"), end="")
	print("Hardware: " + han.get("PRIVate:SET:PCB?"), end="")
	print("Hardware Info: " + han.get("PRIVate:HARDWare:INFO"))
	han.set(":SYSTem:SET:TIME " + datetime.datetime.now().strftime("%Y;%m;%d;%H;%M;%S;"))
	print("System Time: " + han.get("SYSTem:TIME?"))
	print("-----------------")

def corrValue(freq, dB):
	# return str(float(dB))
	for i in range(len(corrlist)):
		thisval = corrlist[i]
		if corrValUnit(thisval["fr"], thisval["un"]) > freq:
			prevval = corrlist[i - 1]
			frdiff = (freq - corrValUnit(prevval["fr"], prevval["un"])) / (
				corrValUnit(thisval["fr"], thisval["un"])
				- corrValUnit(prevval["fr"], prevval["un"]))
			dbdiff = float(thisval["db"]) - float(prevval["db"])
			corr = float(prevval["db"]) + (dbdiff * frdiff)
			return str(float(dB) - corr)
	return "0"	

FN = timeString()
maxDB = -100
minDB = 100
DryRun = False
LoadResistor = False
SignalGenerator = "internal"
SystemInfo = False
NoAutoVolt = False
OutputUnit = "kHz"
DUTCh = 1
REFCh = 0
OneChannel = True
AdjustmentFile = None
Logamode = False
corrlist = []

parseArgs()

print("SG: " + SignalGenerator)

vals, units = HzToValUnit(FS)
vale, unite = HzToValUnit(FE)
if units == "Hz" and unite == "MHz":
	OutputUnit = "kHz"
else:
	OutputUnit = unite

print("Sweep : " + str(vals) + units + " -> " + str(vale) + unite)
print("Points: " + str(PTS))
print("Output Unit: " + OutputUnit)
print("DUT Channel: " + str(DUTCh))
print("Ref Channel: " + str(REFCh))
print("Output Filename: " + str(FN))
if Logamode: print("logarithmic mode")

han = hantek()
if SystemInfo:
	dumpHantekInfo(han)

if PTS < 2:
	print("Error. Points must be >1")
	sys.exit()

han.set(":MEASure:ENABle ON")
han.set(":MEASure:ADISplay OFF")

if SignalGenerator == "fy3200":
	sg = feeltechsg(OneChannel)
elif SignalGenerator == "tinysa":
	sg = tinysasg()
else:
	sg = hanteksg(han)
print(sg.getModel())

if (sg.maxfreq < FE) or (sg.maxfreq < FS) : 
	print("Error. The selected SG can not produce "+HzToString(FE))
	sys.exit()	

sg.setWaveform("sine")

if LoadResistor:
	VPPCh1=VPPCh1 * 2
	if not OneChannel: VPPCh2=VPPCh2 * 2	

if sg.maxvpp<VPPCh2: VPPCh2=sg.maxvpp

if not NoAutoVolt:
	sg.setVPP(VPPCh1, VPPCh2)

if AdjustmentFile is not None:
	with open(AdjustmentFile) as f:
		for line in f:
			entries = line.strip().split(" ", 3)
			corrlist.append({"fr": entries[0], "un": entries[1], "db": entries[2]})
	f.close()
	print("Using Adjustment data from: " + AdjustmentFile)

han.set(":CHANnel" + str(DUTCh) + ":DISPlay ON")
if not NoAutoVolt:
	han.set(":CHANnel" + str(DUTCh) + ":SCALe " + VDIVDUTCh)
han.set(":CHANnel" + str(DUTCh) + ":OFFSet 0")
if not OneChannel:
	han.set(":CHANnel" + str(REFCh) + ":DISPlay ON")
	if not NoAutoVolt:
		han.set(":CHANnel" + str(REFCh) + ":SCALe " + VDIVREFCh)
	han.set(":CHANnel" + str(REFCh) + ":OFFSet 0")
else:
	if DUTCh == 1:
		han.set(":CHANnel2:DISPlay OFF")
	if DUTCh == 2:
		han.set(":CHANnel1:DISPlay OFF")

han.set(":TRIGger:MODE EDGE")
if OneChannel:
	han.set(":TRIGger:EDGe:SOURce CHANnel" + str(DUTCh))
else:
	han.set(":TRIGger:EDGe:SOURce CHANnel" + str(REFCh))
han.set(":TRIGger:EDGe:LEVel 0")

han.set(":TIMebase:SCALe " + str(FreqToVert(FS)))

if not DryRun:
	f = open((FN) + ".dat", "w")

if Logamode:
	points = np.logspace(np.log10(FS), np.log10(FE), num=PTS)
else:
	points = np.linspace(FS, FE, num=PTS)

for x in points:
	currF = x
	han.set(":RUNning ON")
	han.set(":TIMebase:SCALe " + str(FreqToVert(currF)))
	sg.setFrequency(currF)

	time.sleep(SLEEPTIME)
	
	han.set(":RUNning OFF")

	# mFreq = han.measure("1", "FREQuency") #dummy
	mVPPDUT = han.measure(str(DUTCh), "VPP")
	if not OneChannel:
		mPeri = float(han.measure(str(REFCh), "PERIod"))
		if mPeri == 0:
			continue
		mFRR = float(han.measure(str(REFCh), "FRR"))
		Phase = CalcPhase(mFRR, mPeri)
		mVPPRef = han.measure(str(REFCh), "VPP")
	else:
		mFRR = -1
		Phase = -0
		mPeri = 1
		mVPPRef = VPPCh2

	dB = float(V1V2ToDB(mVPPDUT, mVPPRef))
	if AdjustmentFile is not None:
		dB = float(corrValue(currF, dB))

	if mPeri > 0:
		if dB > maxDB:
			maxDB = dB
		if dB < minDB:
			minDB = dB
	if OneChannel:
		print(HzToString(currF) + " | " + str(dB) + "dB | " +
		FormatVolt(mVPPRef) + "V | " + FormatVolt(mVPPDUT) + "V ")
		AVal = HzUnitToVal(currF, OutputUnit)
		if not DryRun:
			f.write( str(AVal) + " " + OutputUnit + " " + str(dB) + " " + str(Phase) + "\n" )
	else:
		print(HzToString(round(1 / mPeri)) + " | " + str(dB) + "dB | " + str(Phase) + "° | " + 
		FormatVolt(mVPPRef) + "V | " + FormatVolt(mVPPDUT) + "V ")
		AVal = HzUnitToVal(round(1 / mPeri), OutputUnit)
		if not DryRun:
			f.write(str(AVal) + " " + OutputUnit + " " + str(dB) + " " + str(Phase) + "\n")

if not DryRun:
	f.close()
han.set(":RUNning ON")

if DryRun:
	sys.exit()

with open(r"plt.tpl", "r") as file:
	data = file.read()
	data = data.replace("%y1rangea%", str(round(minDB) - 5), 1)
	data = data.replace("%y1rangeb%", str(round(maxDB) + 5), 1)
	data = data.replace("%x1rangea%", str(HzUnitToVal(FS, OutputUnit)), 1)
	data = data.replace("%x1rangeb%", str(HzUnitToVal(FE, OutputUnit)), 1)
	data = data.replace("%xunit%", OutputUnit, 1)
	data = data.replace("%devices%", "Hantek DSO2D15 | "+SignalGenerator, 1)
	data = data.replace("%filen%", FN + ".dat")
	if Logamode:
		data = data.replace("%logscale%", "set logscale x 1.2", 1)
	else:
		data = data.replace("%logscale%", "", 1)
with open((FN) + ".plt", "w") as file:
	file.write(data)

os.system("gnuplot -p " + FN + ".plt")

