#!/usr/bin/env python3
import serial
import math
import struct
import sys
import os
import time
import datetime
import pyvisa
from abc import ABC, abstractmethod
from serial.tools import list_ports

# optional import:
import feeltech


class siggen(ABC):  # Abstract base class. Create a class for your SigGen
	def __init__(self, initval=None):
		self._instance = None
		self.maxvpp = 1

	@abstractmethod
	def setWaveform(self, value):
		pass

	@abstractmethod
	def setFrequency(self, value):
		pass

	@abstractmethod
	def setVPP(self, vch1, vch2):
		pass

	@abstractmethod
	def getModel(self, value):
		pass

##### HANTEK SG

class hanteksg(siggen):
	def __init__(self, initval):
		self._instance = initval
		self._instance.set(":DDS:SWITch ON")
		self.maxvpp = 3.5
		self.maxfreq = 2.5e+7

	def setWaveform(self, value):
		if value.lower() == "sine":
			self._instance.set(":DDS:TYPE SINE")

	def setFrequency(self, value):
		self._instance.set(":DDS:FREQ " + str(value))

	def setVPP(self, vch1, vch2):
		self._instance.set(":DDS:AMP " + str(vch1))

	def getModel(self):
		return "Hanktek internal: " + self._instance.get("PRIVate:SET:Model?")

##### FEELTECH SG

class feeltechsg(siggen):
	def __init__(self, OneChannel=False, devPath="/dev/ttyUSB0"):
		self._instance = feeltech.FeelTech(devPath)
		self.maxvpp = 20
		self.maxfreq = 2.4e+7
		if OneChannel:
			self._instance.send("dg0")
		else:
			self._instance.send("dg1")
		self.ch1 = self._instance.channels()[0]
		self.ch2 = self._instance.channels()[1]

	def setWaveform(self, value):
		if value == "sine":
			self.ch1.waveform(feeltech.SINE)
			self.ch2.waveform(feeltech.SINE)

	def setFrequency(self, value):
		self.ch1.frequency(value)
		self.ch2.frequency(value)

	def setVPP(self, vch1, vch2):
		self.ch1.amplitude(vch1)
		self.ch2.amplitude(vch2)

	def getModel(self):
		return "Feeltech: " + self._instance.type()

##### TINYSA SG

class tinysasg(siggen):
	def __init__(self, dev=None):
		self.VID = 0x0483  # 1155
		self.PID = 0x5740  # 22336
		self.maxvpp = 0.75
		self.maxfreq = 3.5e+8
		self.dev = self.getport()
		self.serial = None
		self._frequencies = None
		self.set_low_output()
		time.sleep(0.5)
		self.set_output(True)
		time.sleep(0.5)

	def getport(self) -> str:
		device_list = list_ports.comports()
		for device in device_list:
			if device.vid == self.VID and device.pid == self.PID:
				return device.device
		raise OSError("device not found")

	@property
	def frequencies(self):
		return self._frequencies

	def open(self):
		if self.serial is None:
			self.serial = serial.Serial(self.dev)

	def close(self):
		if self.serial:
			self.serial.close()
		self.serial = None

	def send_command(self, cmd):
		self.open()
		self.serial.write(cmd.encode())
		self.serial.readline()

	def cmd(self, text):
		self.open()
		self.serial.write((text + "\r").encode())
		self.serial.readline()
		data = self.fetch_data()
		return data

	def set_sweep_cw(self, freq):
		if freq is not None:
			self.send_command("sweep cw %s\r" % str(freq))

	def set_level(self, level):
		if level is not None:
			self.send_command("level %d\r" % level)

	def set_output(self, on):
		if on is not None:
			if on:
				self.send_command("output on\r")
			else:
				self.send_command("output off\r")

	def set_low_output(self):
		self.send_command("mode low output\r")

	def fetch_data(self):
		result = ""
		line = ""
		while True:
			c = self.serial.read().decode("utf-8")
			if c == chr(13):
				next  # ignore CR
			line += c
			if c == chr(10):
				result += line
				line = ""
				next
			if line.endswith("ch>"):
				# stop on prompt
				break
		return result

	def setWaveform(self, value):
		self.waveform = value

	def setFrequency(self, value):
		self.set_sweep_cw(value)

	def setVPP(self, vch1, vch2):
		self.set_level(-7)

	def getModel(self):
		return "Tinysa: " + self.cmd("version")

##### HANTEK OSCI

class hantek:
	def __init__(self, dev=None):
		self._instance = self.getDSO()

	def getDSO(self):
		res = pyvisa.ResourceManager("@py")
		listr = res.list_resources("USB0::1183::20574:?*")
		if len(listr) != 1:
			sys.stderr.write("Device with USB ID not found or several found.\n")
			sys.exit(1)
		return res.open_resource(listr[0])

	def get(self, item):
		try:
			res = self._instance.query(item)
			return res
		except pyvisa.errors.VisaIOError as c:
			if type(c) == str:
				if c.find("VI_ERROR_TMO") == -1:
					return c

	def set(self, item):
		try:
			self._instance.write(item)
		except pyvisa.errors.VisaIOError as c:
			if type(c) == str:
				if c.find("VI_ERROR_TMO") == -1:
					return c

	def measure(self, ch, item):
		return self.get(":MEASure:CHANnel" + str(ch) + ":ITEM? " + item)

##### LIBRARY

def timeString():
	cur = datetime.datetime.now()
	return (
		(str(cur.date())).replace("-", "_") + "_" + str(cur.hour) + "_" + str(cur.minute) + "_" + str(cur.second))


def V1V2ToDB(V1, V2):
	return f"{(20*math.log10(float(V1)/float(V2))):.2f}"


def FormatVolt(V):
	return f"{float(V):.3f}"


def StringToHz(s):
	s = s.lower()
	if "m" in s:
		s = s.partition("m")[0]
		return float(s) * 1000000
	elif "k" in s:
		s = s.partition("k")[0]
		return float(s) * 1000
	elif "h" in s:
		s = s.partition("h")[0]
		return float(s)
	else:
		return float(s)


def HzToValUnit(Hz: float):
	if float(Hz) < 1000:
		return Hz, "Hz"
	if float(Hz) < 1000000:
		return Hz / 1000, "kHz"
	if float(Hz) < 1000000000:
		return Hz / 1000000, "MHz"


def HzToString(Hz: float):
	val, unit = HzToValUnit(Hz)
	return str(val) + unit


def HzUnitToVal(Hz: float, Unit: str):
	if Unit == "Hz":
		return Hz
	elif Unit == "kHz":
		return Hz / 1000
	elif Unit == "MHz":
		return Hz / 1000000
	else:
		return Hz


def FreqToVert(freq):
	timebase = 1 / (freq / 2)
	for i in range(16):
		if timebase < 1 / 10**i and timebase > 1 / 10 ** (i + 1):
			rng = timebase * 10 ** (i + 1)
			if rng < 1: res = 1
			elif rng < 2: res = 2
			elif rng < 5: res = 5
			else:res = 10
			return res * 1 / 10 ** (i + 2)


def CalcPhase(a, b, deg=180):
	af = float(a)
	bf = float(b)
	if af < 0:
		af += bf
	if bf == 0:
		return 0
	divi = af / bf
	if divi>1: divi=1
	return round(divi * 360) - 180


def corrValUnit(freq, Unit="MHz"):
	if Unit == "Hz":
		return float(freq)
	elif Unit == "kHz":
		return float(freq) * 1000
	elif Unit == "MHz":
		return float(freq) * 1000000
	else:
		return float(freq)
